@@ -13,48 +13,49 @@
     const EventEmitter = NativeModule.require('events');
     process._eventsCount = 0;
 
-    Object.setPrototypeOf(process, Object.create(EventEmitter.prototype, {
-      constructor: {
-        value: process.constructor
-      }
-    }));
+    const origProcProto = Object.getPrototypeOf(process);
+    Object.setPrototypeOf(origProcProto, EventEmitter.prototype);
 
     EventEmitter.call(process);
 
-    let eeWarned = false;
-    Object.defineProperty(process, 'EventEmitter', {
-      get() {
-        const internalUtil = NativeModule.require('internal/util');
-        eeWarned = internalUtil.printDeprecationMessage(
-          "process.EventEmitter is deprecated. Use require('events') instead.",
-          eeWarned
-        );
-        return EventEmitter;
-      }
-    });
-
     setupProcessObject();
 
     // do this good and early, since it handles errors.
     setupProcessFatal();
 
+    setupProcessICUVersions();
+
     setupGlobalVariables();
-    if (!process._noBrowserGlobals) {
-      setupGlobalTimeouts();
-      setupGlobalConsole();
-    }
 
     const _process = NativeModule.require('internal/process');
-
-    _process.setup_hrtime();
-    _process.setup_cpuUsage();
-    _process.setupMemoryUsage();
     _process.setupConfig(NativeModule._source);
+    _process.setupSignalHandlers();
     NativeModule.require('internal/process/warning').setup();
     NativeModule.require('internal/process/next_tick').setup();
     NativeModule.require('internal/process/stdio').setup();
+
+    const perf = process.binding('performance');
+    const {
+      NODE_PERFORMANCE_MILESTONE_BOOTSTRAP_COMPLETE,
+      NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_START,
+      NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_END,
+      NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_START,
+      NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_END,
+      NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START,
+      NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END,
+      NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START,
+      NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END
+    } = perf.constants;
+
+    _process.setup_hrtime();
+    _process.setup_performance();
+    _process.setup_cpuUsage();
+    _process.setupMemoryUsage();
     _process.setupKillAndExit();
-    _process.setupSignalHandlers();
+    if (global.__coverage__)
+      NativeModule.require('internal/process/write-coverage').setup();
+
+    NativeModule.require('internal/inspector_async_hook').setup();
 
     // Do not initialize channel in debugger agent, it deletes env variable
     // and the main thread won't see it.
@@ -63,6 +64,16 @@
 
     _process.setupRawDebug();
 
+    const browserGlobals = !process._noBrowserGlobals;
+    if (browserGlobals) {
+      setupGlobalTimeouts();
+      setupGlobalConsole();
+    }
+
+    // Ensure setURLConstructor() is called before the native
+    // URL::ToObject() method is used.
+    NativeModule.require('internal/url');
+
     Object.defineProperty(process, 'argv0', {
       enumerable: true,
       configurable: false,
@@ -70,6 +81,27 @@
     });
     process.argv[0] = process.execPath;
 
+    // Handle `--debug*` deprecation and invalidation
+    if (process._invalidDebug) {
+      process.emitWarning(
+        '`node --debug` and `node --debug-brk` are invalid. ' +
+        'Please use `node --inspect` or `node --inspect-brk` instead.',
+        'DeprecationWarning', 'DEP0062', startup, true);
+      process.exit(9);
+    } else if (process._deprecatedDebugBrk) {
+      process.emitWarning(
+        '`node --inspect --debug-brk` is deprecated. ' +
+        'Please use `node --inspect-brk` instead.',
+        'DeprecationWarning', 'DEP0062', startup, true);
+    }
+
+    if (process.binding('config').experimentalModules) {
+      process.emitWarning(
+        'The ESM module loader is experimental.',
+        'ExperimentalWarning', undefined);
+    }
+
+
     // There are various modes that Node can run in. The most common two
     // are running from a script and running the REPL - but there are a few
     // others like the debugger or running --eval arguments. Here we decide
@@ -80,20 +112,22 @@
       // one to drop a file lib/_third_party_main.js into the build
       // directory which will be executed instead of Node's normal loading.
       process.nextTick(function() {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_START);
         NativeModule.require('_third_party_main');
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_THIRD_PARTY_MAIN_END);
       });
 
-    } else if (process.argv[1] === 'debug') {
-      // Start the debugger agent
-      NativeModule.require('_debugger').start();
+    } else if (process.argv[1] === 'inspect' || process.argv[1] === 'debug') {
+      if (process.argv[1] === 'debug') {
+        process.emitWarning(
+          '`node debug` is deprecated. Please use `node inspect` instead.',
+          'DeprecationWarning', 'DEP0068');
+      }
 
-    } else if (process.argv[1] === '--remote_debugging_server') {
-      // Start the debugging server
-      NativeModule.require('internal/inspector/remote_debugging_server');
-
-    } else if (process.argv[1] === '--debug-agent') {
       // Start the debugger agent
-      NativeModule.require('_debug_agent').start();
+      process.nextTick(function() {
+        NativeModule.require('node-inspect/lib/_inspect').start();
+      });
 
     } else if (process.profProcess) {
       NativeModule.require('internal/v8_prof_processor');
@@ -105,24 +139,30 @@
       // channel. This needs to be done before any user code gets executed
       // (including preload modules).
       if (process.argv[1] && process.env.NODE_UNIQUE_ID) {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_START);
         const cluster = NativeModule.require('cluster');
         cluster._setupWorker();
-
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_CLUSTER_SETUP_END);
         // Make sure it's not accidentally inherited by child processes.
         delete process.env.NODE_UNIQUE_ID;
       }
 
       if (process._eval != null && !process._forceRepl) {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START);
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END);
         // User passed '-e' or '--eval' arguments to Node without '-i' or
         // '--interactive'
+
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START);
         preloadModules();
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END);
 
         const internalModule = NativeModule.require('internal/module');
         internalModule.addBuiltinLibsToObject(global);
-        run(() => {
-          evalScript('[eval]');
-        });
-      } else if (process.argv[1]) {
+        evalScript('[eval]');
+      } else if (process.argv[1] && process.argv[1] !== '-') {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START);
         // make process.argv[1] into a full path
         const path = NativeModule.require('path');
         process.argv[1] = path.resolve(process.argv[1]);
@@ -131,25 +171,28 @@
 
         // check if user passed `-c` or `--check` arguments to Node.
         if (process._syntax_check_only != null) {
-          const vm = NativeModule.require('vm');
           const fs = NativeModule.require('fs');
-          const internalModule = NativeModule.require('internal/module');
           // read the source
           const filename = Module._resolveFilename(process.argv[1]);
           var source = fs.readFileSync(filename, 'utf-8');
-          // remove shebang and BOM
-          source = internalModule.stripBOM(source.replace(/^#!.*/, ''));
-          // wrap it
-          source = Module.wrap(source);
-          // compile the script, this will throw if it fails
-          new vm.Script(source, {filename: filename, displayErrors: true});
+          checkScriptSyntax(source, filename);
           process.exit(0);
         }
-
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END);
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START);
         preloadModules();
-        run(Module.runMain);
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END);
+        Module.runMain();
       } else {
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_START);
+        perf.markMilestone(NODE_PERFORMANCE_MILESTONE_MODULE_LOAD_END);
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_START);
         preloadModules();
+        perf.markMilestone(
+          NODE_PERFORMANCE_MILESTONE_PRELOAD_MODULE_LOAD_END);
         // If -i or --interactive were passed, or stdin is a TTY.
         if (process._forceRepl || NativeModule.require('tty').isatty(0)) {
           // REPL
@@ -183,12 +226,17 @@
           });
 
           process.stdin.on('end', function() {
-            process._eval = code;
-            evalScript('[stdin]');
+            if (process._syntax_check_only != null) {
+              checkScriptSyntax(code, '[stdin]');
+            } else {
+              process._eval = code;
+              evalScript('[stdin]');
+            }
           });
         }
       }
     }
+    perf.markMilestone(NODE_PERFORMANCE_MILESTONE_BOOTSTRAP_COMPLETE);
   }
 
   function setupProcessObject() {
@@ -201,26 +249,43 @@
   }
 
   function setupGlobalVariables() {
+    Object.defineProperty(global, Symbol.toStringTag, {
+      value: 'global',
+      writable: false,
+      enumerable: false,
+      configurable: true
+    });
     global.process = process;
     const util = NativeModule.require('util');
 
-    // Deprecate GLOBAL and root
-    ['GLOBAL', 'root'].forEach(function(name) {
-      // getter
-      const get = util.deprecate(function() {
+    function makeGetter(name) {
+      return util.deprecate(function() {
         return this;
-      }, `'${name}' is deprecated, use 'global'`);
-      // setter
-      const set = util.deprecate(function(value) {
+      }, `'${name}' is deprecated, use 'global'`, 'DEP0016');
+    }
+
+    function makeSetter(name) {
+      return util.deprecate(function(value) {
         Object.defineProperty(this, name, {
           configurable: true,
           writable: true,
           enumerable: true,
           value: value
         });
-      }, `'${name}' is deprecated, use 'global'`);
-      // define property
-      Object.defineProperty(global, name, { get, set, configurable: true });
+      }, `'${name}' is deprecated, use 'global'`, 'DEP0016');
+    }
+
+    Object.defineProperties(global, {
+      GLOBAL: {
+        configurable: true,
+        get: makeGetter('GLOBAL'),
+        set: makeSetter('GLOBAL')
+      },
+      root: {
+        configurable: true,
+        get: makeGetter('root'),
+        set: makeSetter('root')
+      }
     });
 
     global.Buffer = NativeModule.require('buffer').Buffer;
@@ -239,59 +304,71 @@
   }
 
   function setupGlobalConsole() {
-    var inspectorConsole;
-    var wrapConsoleCall;
-    if (process.inspector) {
-      inspectorConsole = global.console;
-      wrapConsoleCall = process.inspector.wrapConsoleCall;
-      delete process.inspector;
-    }
-    var console;
+    const originalConsole = global.console;
+    const Module = NativeModule.require('module');
+    // Setup Node.js global.console
+    const wrappedConsole = NativeModule.require('console');
     Object.defineProperty(global, 'console', {
       configurable: true,
       enumerable: true,
-      get: function() {
-        if (!console) {
-          console = NativeModule.require('console');
-          installInspectorConsoleIfNeeded(console,
-                                          inspectorConsole,
-                                          wrapConsoleCall);
-        }
-        return console;
+      get() {
+        return wrappedConsole;
       }
     });
+    setupInspector(originalConsole, wrappedConsole, Module);
   }
 
-  function installInspectorConsoleIfNeeded(console,
-                                           inspectorConsole,
-                                           wrapConsoleCall) {
-    if (!inspectorConsole)
+  function setupInspector(originalConsole, wrappedConsole, Module) {
+    if (!process.config.variables.v8_enable_inspector) {
       return;
+    }
+    const { addCommandLineAPI, consoleCall } = process.binding('inspector');
+    // Setup inspector command line API
+    const { makeRequireFunction } = NativeModule.require('internal/module');
+    const path = NativeModule.require('path');
+    const cwd = tryGetCwd(path);
+
+    const consoleAPIModule = new Module('<inspector console>');
+    consoleAPIModule.paths =
+        Module._nodeModulePaths(cwd).concat(Module.globalPaths);
+    addCommandLineAPI('require', makeRequireFunction(consoleAPIModule));
     const config = {};
-    for (const key of Object.keys(console)) {
-      if (!inspectorConsole.hasOwnProperty(key))
+    for (const key of Object.keys(wrappedConsole)) {
+      if (!originalConsole.hasOwnProperty(key))
         continue;
-      // If node console has the same method as inspector console,
+      // If global console has the same method as inspector console,
       // then wrap these two methods into one. Native wrapper will preserve
       // the original stack.
-      console[key] = wrapConsoleCall(inspectorConsole[key],
-                                     console[key],
-                                     config);
+      wrappedConsole[key] = consoleCall.bind(wrappedConsole,
+                                             originalConsole[key],
+                                             wrappedConsole[key],
+                                             config);
     }
-    for (const key of Object.keys(inspectorConsole)) {
-      if (console.hasOwnProperty(key))
+    for (const key of Object.keys(originalConsole)) {
+      if (wrappedConsole.hasOwnProperty(key))
         continue;
-      console[key] = inspectorConsole[key];
+      wrappedConsole[key] = originalConsole[key];
     }
   }
 
   function setupProcessFatal() {
+    const async_wrap = process.binding('async_wrap');
+    // Arrays containing hook flags and ids for async_hook calls.
+    const { async_hook_fields, async_id_fields } = async_wrap;
+    // Internal functions needed to manipulate the stack.
+    const { clearAsyncIdStack, asyncIdStackSize } = async_wrap;
+    const { kAfter, kExecutionAsyncId,
+            kInitTriggerAsyncId } = async_wrap.constants;
 
     process._fatalException = function(er) {
       var caught;
 
+      // It's possible that kInitTriggerAsyncId was set for a constructor call
+      // that threw and was never cleared. So clear it now.
+      async_id_fields[kInitTriggerAsyncId] = 0;
+
       if (process.domain && process.domain._errorHandler)
-        caught = process.domain._errorHandler(er) || caught;
+        caught = process.domain._errorHandler(er);
 
       if (!caught)
         caught = process.emit('uncaughtException', er);
@@ -308,30 +385,61 @@
           // nothing to be done about it at this point.
         }
 
-      // if we handled an error, then make sure any ticks get processed
       } else {
+        // If we handled an error, then make sure any ticks get processed
         NativeModule.require('timers').setImmediate(process._tickCallback);
+
+        // Emit the after() hooks now that the exception has been handled.
+        if (async_hook_fields[kAfter] > 0) {
+          do {
+            NativeModule.require('async_hooks').emitAfter(
+              async_id_fields[kExecutionAsyncId]);
+          } while (asyncIdStackSize() > 0);
+        // Or completely empty the id stack.
+        } else {
+          clearAsyncIdStack();
+        }
       }
 
       return caught;
     };
   }
 
-  function tryGetCwd(path) {
-    var threw = true;
-    var cwd;
-    try {
-      cwd = process.cwd();
-      threw = false;
-    } finally {
-      if (threw) {
-        // getcwd(3) can fail if the current working directory has been deleted.
-        // Fall back to the directory name of the (absolute) executable path.
-        // It's not really correct but what are the alternatives?
-        return path.dirname(process.execPath);
-      }
+  function setupProcessICUVersions() {
+    const icu = process.binding('config').hasIntl ?
+      process.binding('icu') : undefined;
+    if (!icu) return;  // no Intl/ICU: nothing to add here.
+    // With no argument, getVersion() returns a comma separated list
+    // of possible types.
+    const versionTypes = icu.getVersion().split(',');
+
+    for (var n = 0; n < versionTypes.length; n++) {
+      var name = versionTypes[n];
+      const version = icu.getVersion(name);
+      Object.defineProperty(process.versions, name, {
+        writable: false,
+        enumerable: true,
+        value: version
+      });
     }
-    return cwd;
+  }
+
+  function tryGetCwd(path) {
+    try {
+      return process.cwd();
+    } catch (ex) {
+      // getcwd(3) can fail if the current working directory has been deleted.
+      // Fall back to the directory name of the (absolute) executable path.
+      // It's not really correct but what are the alternatives?
+      return path.dirname(process.execPath);
+    }
+  }
+
+  function wrapForBreakOnFirstLine(source) {
+    if (!process._breakFirstLine)
+      return source;
+    const fn = `function() {\n\n${source};\n\n}`;
+    return `process.binding('inspector').callAndPauseOnStart(${fn}, {})`;
   }
 
   function evalScript(name) {
@@ -342,7 +450,7 @@
     const module = new Module(name);
     module.filename = path.join(cwd, name);
     module.paths = Module._nodeModulePaths(cwd);
-    const body = process._eval;
+    const body = wrapForBreakOnFirstLine(process._eval);
     const script = `global.__filename = ${JSON.stringify(name)};\n` +
                    'global.exports = exports;\n' +
                    'global.module = module;\n' +
@@ -351,13 +459,10 @@
                    'return require("vm").runInThisContext(' +
                    `${JSON.stringify(body)}, { filename: ` +
                    `${JSON.stringify(name)}, displayErrors: true });\n`;
-    // Defer evaluation for a tick.  This is a workaround for deferred
-    // events not firing when evaluating scripts from the command line,
-    // see https://github.com/nodejs/node/issues/1600.
-    setImmediate(function() {
-      const result = module._compile(script, `${name}-wrapper`);
-      if (process._print_eval) console.log(result);
-    });
+    const result = module._compile(script, `${name}-wrapper`);
+    if (process._print_eval) console.log(result);
+    // Handle any nextTicks added in the first tick of the program.
+    process._tickCallback();
   }
 
   // Load preload modules
@@ -367,31 +472,19 @@
     }
   }
 
-  function isDebugBreak() {
-    return process.execArgv.some((arg) => /^--debug-brk(=[0-9]+)?$/.test(arg));
-  }
+  function checkScriptSyntax(source, filename) {
+    const Module = NativeModule.require('module');
+    const vm = NativeModule.require('vm');
+    const internalModule = NativeModule.require('internal/module');
 
-  function run(entryFunction) {
-    if (process._debugWaitConnect && isDebugBreak()) {
-
-      // XXX Fix this terrible hack!
-      //
-      // Give the client program a few ticks to connect.
-      // Otherwise, there's a race condition where `node debug foo.js`
-      // will not be able to connect in time to catch the first
-      // breakpoint message on line 1.
-      //
-      // A better fix would be to somehow get a message from the
-      // V8 debug object about a connection, and runMain when
-      // that occurs.  --isaacs
-
-      const debugTimeout = +process.env.NODE_DEBUG_TIMEOUT || 50;
-      setTimeout(entryFunction, debugTimeout);
-
-    } else {
-      // Main entry point into most programs:
-      entryFunction();
-    }
+    // remove Shebang
+    source = internalModule.stripShebang(source);
+    // remove BOM
+    source = internalModule.stripBOM(source);
+    // wrap it
+    source = Module.wrap(source);
+    // compile the script, this will throw if it fails
+    new vm.Script(source, { displayErrors: true, filename });
   }
 
   // Below you find a minimal module system, which is used to load the node
@@ -415,6 +508,11 @@
   NativeModule._source = process.binding('natives');
   NativeModule._cache = {};
 
+  const config = process.binding('config');
+
+  if (!config.exposeHTTP2)
+    delete NativeModule._source.http2;
+
   NativeModule.require = function(id) {
     if (id === 'native_module') {
       return NativeModule;
@@ -426,7 +524,13 @@
     }
 
     if (!NativeModule.exists(id)) {
-      throw new Error(`No such native module ${id}`);
+      // Model the error off the internal/errors.js model, but
+      // do not use that module given that it could actually be
+      // the one causing the error if there's a bug in Node.js
+      const err = new Error(`No such built-in module: ${id}`);
+      err.code = 'ERR_UNKNOWN_BUILTIN_MODULE';
+      err.name = 'Error [ERR_UNKNOWN_BUILTIN_MODULE]';
+      throw err;
     }
 
     process.moduleLoadList.push(`NativeModule ${id}`);
@@ -447,11 +551,7 @@
     return NativeModule._source.hasOwnProperty(id);
   };
 
-  const EXPOSE_INTERNALS = process.execArgv.some(function(arg) {
-    return arg.match(/^--expose[-_]internals$/);
-  });
-
-  if (EXPOSE_INTERNALS) {
+  if (config.exposeInternals) {
     NativeModule.nonInternalExists = NativeModule.exists;
 
     NativeModule.isInternal = function(id) {
