@@ -1,24 +1,51 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
 const NativeModule = require('native_module');
 const util = require('util');
 const internalModule = require('internal/module');
-const internalUtil = require('internal/util');
+const { getURLFromFilePath } = require('internal/url');
 const vm = require('vm');
 const assert = require('assert').ok;
 const fs = require('fs');
+const internalFS = require('internal/fs');
 const path = require('path');
-const internalModuleReadFile = process.binding('fs').internalModuleReadFile;
-const internalModuleStat = process.binding('fs').internalModuleStat;
+const {
+  internalModuleReadFile,
+  internalModuleStat
+} = process.binding('fs');
 const preserveSymlinks = !!process.binding('config').preserveSymlinks;
+const experimentalModules = !!process.binding('config').experimentalModules;
 
-// If obj.hasOwnProperty has been overridden, then calling
-// obj.hasOwnProperty(prop) will break.
-// See: https://github.com/joyent/node/issues/1707
-function hasOwnProperty(obj, prop) {
-  return Object.prototype.hasOwnProperty.call(obj, prop);
-}
+const errors = require('internal/errors');
 
+module.exports = Module;
+
+// these are below module.exports for the circular reference
+const Loader = require('internal/loader/Loader');
+const ModuleJob = require('internal/loader/ModuleJob');
+const { createDynamicModule } = require('internal/loader/ModuleWrap');
+let ESMLoader;
 
 function stat(filename) {
   filename = path._makeLong(filename);
@@ -33,24 +60,25 @@ function stat(filename) {
 }
 stat.cache = null;
 
+function updateChildren(parent, child, scan) {
+  var children = parent && parent.children;
+  if (children && !(scan && children.includes(child)))
+    children.push(child);
+}
 
 function Module(id, parent) {
   this.id = id;
   this.exports = {};
   this.parent = parent;
-  if (parent && parent.children) {
-    parent.children.push(this);
-  }
-
+  updateChildren(parent, this, false);
   this.filename = null;
   this.loaded = false;
   this.children = [];
 }
-module.exports = Module;
 
-Module._cache = {};
-Module._pathCache = {};
-Module._extensions = {};
+Module._cache = Object.create(null);
+Module._pathCache = Object.create(null);
+Module._extensions = Object.create(null);
 var modulePaths = [];
 Module.globalPaths = [];
 
@@ -74,12 +102,12 @@ const debug = Module._debug;
 //   -> a/index.<ext>
 
 // check if the directory is a package.json dir
-const packageMainCache = {};
+const packageMainCache = Object.create(null);
 
 function readPackage(requestPath) {
-  if (hasOwnProperty(packageMainCache, requestPath)) {
-    return packageMainCache[requestPath];
-  }
+  const entry = packageMainCache[requestPath];
+  if (entry)
+    return entry;
 
   const jsonPath = path.resolve(requestPath, 'package.json');
   const json = internalModuleReadFile(path._makeLong(jsonPath));
@@ -114,9 +142,6 @@ function tryPackage(requestPath, exts, isMain) {
 // Set to an empty Map to reset.
 const realpathCache = new Map();
 
-const realpathCacheKey = fs.realpathCacheKey;
-delete fs.realpathCacheKey;
-
 // check if the file exists and is not a directory
 // if using --preserve-symlinks and isMain is false,
 // keep symlinks intact, otherwise resolve to the
@@ -131,11 +156,11 @@ function tryFile(requestPath, isMain) {
 
 function toRealPath(requestPath) {
   return fs.realpathSync(requestPath, {
-    [realpathCacheKey]: realpathCache
+    [internalFS.realpathCacheKey]: realpathCache
   });
 }
 
-// given a path check a the file exists with any of the set extensions
+// given a path, check if the file exists with any of the set extensions
 function tryExtensions(p, exts, isMain) {
   for (var i = 0; i < exts.length; i++) {
     const filename = tryFile(p + exts[i], isMain);
@@ -155,14 +180,15 @@ Module._findPath = function(request, paths, isMain) {
     return false;
   }
 
-  const cacheKey = JSON.stringify({request: request, paths: paths});
-  if (Module._pathCache[cacheKey]) {
-    return Module._pathCache[cacheKey];
-  }
+  var cacheKey = request + '\x00' +
+                (paths.length === 1 ? paths[0] : paths.join('\x00'));
+  var entry = Module._pathCache[cacheKey];
+  if (entry)
+    return entry;
 
   var exts;
-  const trailingSlash = request.length > 0 &&
-                        request.charCodeAt(request.length - 1) === 47/*/*/;
+  var trailingSlash = request.length > 0 &&
+                      request.charCodeAt(request.length - 1) === 47/*/*/;
 
   // For each path
   for (var i = 0; i < paths.length; i++) {
@@ -172,8 +198,8 @@ Module._findPath = function(request, paths, isMain) {
     var basePath = path.resolve(curPath, request);
     var filename;
 
+    var rc = stat(basePath);
     if (!trailingSlash) {
-      const rc = stat(basePath);
       if (rc === 0) {  // File.
         if (preserveSymlinks && !isMain) {
           filename = path.resolve(basePath);
@@ -194,26 +220,25 @@ Module._findPath = function(request, paths, isMain) {
       }
     }
 
-    if (!filename) {
+    if (!filename && rc === 1) {  // Directory.
       if (exts === undefined)
         exts = Object.keys(Module._extensions);
-      filename = tryPackage(basePath, exts, isMain);
-    }
-
-    if (!filename) {
-      // try it with each of the extensions at "index"
-      if (exts === undefined)
-        exts = Object.keys(Module._extensions);
-      filename = tryExtensions(path.resolve(basePath, 'index'), exts, isMain);
+      filename = tryPackage(basePath, exts, isMain) ||
+        // try it with each of the extensions at "index"
+        tryExtensions(path.resolve(basePath, 'index'), exts, isMain);
     }
 
     if (filename) {
       // Warn once if '.' resolved outside the module dir
       if (request === '.' && i > 0) {
-        warned = internalUtil.printDeprecationMessage(
-          'warning: require(\'.\') resolved outside the package ' +
-          'directory. This functionality is deprecated and will be removed ' +
-          'soon.', warned);
+        if (!warned) {
+          warned = true;
+          process.emitWarning(
+            'warning: require(\'.\') resolved outside the package ' +
+            'directory. This functionality is deprecated and will be removed ' +
+            'soon.',
+            'DeprecationWarning', 'DEP0019');
+        }
       }
 
       Module._pathCache[cacheKey] = filename;
@@ -250,8 +275,8 @@ if (process.platform === 'win32') {
       // The path segment separator check ('\' and '/') was used to get
       // node_modules path for every path segment.
       // Use colon as an extra condition since we can get node_modules
-      // path for dirver root like 'C:\node_modules' and don't need to
-      // parse driver name.
+      // path for drive root like 'C:\node_modules' and don't need to
+      // parse drive name.
       if (code === 92/*\*/ || code === 47/*/*/ || code === 58/*:*/) {
         if (p !== nmLen)
           paths.push(from.slice(0, last) + '\\node_modules');
@@ -311,14 +336,14 @@ if (process.platform === 'win32') {
 // 'index.' character codes
 var indexChars = [ 105, 110, 100, 101, 120, 46 ];
 var indexLen = indexChars.length;
-Module._resolveLookupPaths = function(request, parent) {
+Module._resolveLookupPaths = function(request, parent, newReturn) {
   if (NativeModule.nonInternalExists(request)) {
-    return [request, []];
+    debug('looking for %j in []', request);
+    return (newReturn ? null : [request, []]);
   }
 
-  var reqLen = request.length;
   // Check for relative path
-  if (reqLen < 2 ||
+  if (request.length < 2 ||
       request.charCodeAt(0) !== 46/*.*/ ||
       (request.charCodeAt(1) !== 46/*.*/ &&
        request.charCodeAt(1) !== 47/*/*/)) {
@@ -340,7 +365,8 @@ Module._resolveLookupPaths = function(request, parent) {
       }
     }
 
-    return [request, paths];
+    debug('looking for %j in %j', request, paths);
+    return (newReturn ? (paths.length > 0 ? paths : null) : [request, paths]);
   }
 
   // with --eval, parent.id is not set and parent.filename is null
@@ -348,7 +374,9 @@ Module._resolveLookupPaths = function(request, parent) {
     // make require('./path/to/foo') work - normally the path is taken
     // from realpath(__filename) but with eval there is no filename
     var mainPaths = ['.'].concat(Module._nodeModulePaths('.'), modulePaths);
-    return [request, mainPaths];
+
+    debug('looking for %j in %j', request, mainPaths);
+    return (newReturn ? mainPaths : [request, mainPaths]);
   }
 
   // Is the parent an index module?
@@ -398,7 +426,9 @@ Module._resolveLookupPaths = function(request, parent) {
   debug('RELATIVE: requested: %s set ID to: %s from %s', request, id,
         parent.id);
 
-  return [id, [path.dirname(parent.filename)]];
+  var parentDir = [path.dirname(parent.filename)];
+  debug('looking for %j in %j', id, parentDir);
+  return (newReturn ? parentDir : [id, parentDir]);
 };
 
 
@@ -414,10 +444,32 @@ Module._load = function(request, parent, isMain) {
     debug('Module._load REQUEST %s parent: %s', request, parent.id);
   }
 
+  if (isMain && experimentalModules) {
+    (async () => {
+      // loader setup
+      if (!ESMLoader) {
+        ESMLoader = new Loader();
+        const userLoader = process.binding('config').userLoader;
+        if (userLoader) {
+          const hooks = await ESMLoader.import(userLoader);
+          ESMLoader = new Loader();
+          ESMLoader.hook(hooks);
+        }
+      }
+      await ESMLoader.import(getURLFromFilePath(request).pathname);
+    })()
+    .catch((e) => {
+      console.error(e);
+      process.exit(1);
+    });
+    return;
+  }
+
   var filename = Module._resolveFilename(request, parent, isMain);
 
   var cachedModule = Module._cache[filename];
   if (cachedModule) {
+    updateChildren(parent, cachedModule, true);
     return cachedModule.exports;
   }
 
@@ -426,6 +478,7 @@ Module._load = function(request, parent, isMain) {
     return NativeModule.require(filename);
   }
 
+  // Don't call updateChildren(), Module constructor already does.
   var module = new Module(filename, parent);
 
   if (isMain) {
@@ -452,21 +505,37 @@ function tryModuleLoad(module, filename) {
   }
 }
 
-Module._resolveFilename = function(request, parent, isMain) {
+Module._resolveFilename = function(request, parent, isMain, options) {
   if (NativeModule.nonInternalExists(request)) {
     return request;
   }
 
-  var resolvedModule = Module._resolveLookupPaths(request, parent);
-  var id = resolvedModule[0];
-  var paths = resolvedModule[1];
+  var paths;
+
+  if (typeof options === 'object' && options !== null &&
+      Array.isArray(options.paths)) {
+    paths = [];
+
+    for (var i = 0; i < options.paths.length; i++) {
+      const path = options.paths[i];
+      const lookupPaths = Module._resolveLookupPaths(path, parent, true);
+
+      if (!paths.includes(path))
+        paths.push(path);
+
+      for (var j = 0; j < lookupPaths.length; j++) {
+        if (!paths.includes(lookupPaths[j]))
+          paths.push(lookupPaths[j]);
+      }
+    }
+  } else {
+    paths = Module._resolveLookupPaths(request, parent, true);
+  }
 
   // look up the filename first, since that's the cache key.
-  debug('looking for %j in %j', id, paths);
-
   var filename = Module._findPath(request, paths, isMain);
   if (!filename) {
-    var err = new Error("Cannot find module '" + request + "'");
+    var err = new Error(`Cannot find module '${request}'`);
     err.code = 'MODULE_NOT_FOUND';
     throw err;
   }
@@ -486,6 +555,27 @@ Module.prototype.load = function(filename) {
   if (!Module._extensions[extension]) extension = '.js';
   Module._extensions[extension](this, filename);
   this.loaded = true;
+
+  if (ESMLoader) {
+    const url = getURLFromFilePath(filename);
+    const urlString = `${url}`;
+    const exports = this.exports;
+    if (ESMLoader.moduleMap.has(urlString) !== true) {
+      ESMLoader.moduleMap.set(
+        urlString,
+        new ModuleJob(ESMLoader, url, async () => {
+          const ctx = createDynamicModule(
+            ['default'], url);
+          ctx.reflect.exports.default.set(exports);
+          return ctx;
+        })
+      );
+    } else {
+      const job = ESMLoader.moduleMap.get(urlString);
+      if (job.reflect)
+        job.reflect.exports.default.set(exports);
+    }
+  }
 };
 
 
@@ -499,7 +589,7 @@ Module.prototype.require = function(path) {
 
 
 // Resolved path to process.argv[1] will be lazily placed here
-// (needed for setting breakpoint when called with --debug-brk)
+// (needed for setting breakpoint when called with --inspect-brk)
 var resolvedArgv;
 
 
@@ -508,33 +598,8 @@ var resolvedArgv;
 // the file.
 // Returns exception, if any.
 Module.prototype._compile = function(content, filename) {
-  // Remove shebang
-  var contLen = content.length;
-  if (contLen >= 2) {
-    if (content.charCodeAt(0) === 35/*#*/ &&
-        content.charCodeAt(1) === 33/*!*/) {
-      if (contLen === 2) {
-        // Exact match
-        content = '';
-      } else {
-        // Find end of shebang line and slice it off
-        var i = 2;
-        for (; i < contLen; ++i) {
-          var code = content.charCodeAt(i);
-          if (code === 10/*\n*/ || code === 13/*\r*/)
-            break;
-        }
-        if (i === contLen)
-          content = '';
-        else {
-          // Note that this actually includes the newline character(s) in the
-          // new output. This duplicates the behavior of the regular expression
-          // that was previously used to replace the shebang line
-          content = content.slice(i);
-        }
-      }
-    }
-  }
+
+  content = internalModule.stripShebang(content);
 
   // create wrapper function
   var wrapper = Module.wrap(content);
@@ -545,11 +610,12 @@ Module.prototype._compile = function(content, filename) {
     displayErrors: true
   });
 
-  if (process._debugWaitConnect && process._eval == null) {
+  var inspectorWrapper = null;
+  if (process._breakFirstLine && process._eval == null) {
     if (!resolvedArgv) {
       // we enter the repl if we're not given a filename argument.
       if (process.argv[1]) {
-        resolvedArgv = Module._resolveFilename(process.argv[1], null);
+        resolvedArgv = Module._resolveFilename(process.argv[1], null, false);
       } else {
         resolvedArgv = 'repl';
       }
@@ -557,17 +623,26 @@ Module.prototype._compile = function(content, filename) {
 
     // Set breakpoint on module start
     if (filename === resolvedArgv) {
-      delete process._debugWaitConnect;
-      const Debug = vm.runInDebugContext('Debug');
-      Debug.setBreakPoint(compiledWrapper, 0, 0);
+      delete process._breakFirstLine;
+      inspectorWrapper = process.binding('inspector').callAndPauseOnStart;
+      if (!inspectorWrapper) {
+        const Debug = vm.runInDebugContext('Debug');
+        Debug.setBreakPoint(compiledWrapper, 0, 0);
+      }
     }
   }
   var dirname = path.dirname(filename);
-  var require = internalModule.makeRequireFunction.call(this);
-  var args = [this.exports, require, this, filename, dirname];
+  var require = internalModule.makeRequireFunction(this);
   var depth = internalModule.requireDepth;
   if (depth === 0) stat.cache = new Map();
-  var result = compiledWrapper.apply(this.exports, args);
+  var result;
+  if (inspectorWrapper) {
+    result = inspectorWrapper(compiledWrapper, this.exports, this.exports,
+                              require, this, filename, dirname);
+  } else {
+    result = compiledWrapper.call(this.exports, this.exports, require, this,
+                                  filename, dirname);
+  }
   if (depth === 0) stat.cache = null;
   return result;
 };
@@ -597,6 +672,11 @@ Module._extensions['.node'] = function(module, filename) {
   return process.dlopen(module, path._makeLong(filename));
 };
 
+if (experimentalModules) {
+  Module._extensions['.mjs'] = function(module, filename) {
+    throw new errors.Error('ERR_REQUIRE_ESM', filename);
+  };
+}
 
 // bootstrap main module.
 Module.runMain = function() {
@@ -616,7 +696,16 @@ Module._initPaths = function() {
     homeDir = process.env.HOME;
   }
 
-  var paths = [path.resolve(process.execPath, '..', '..', 'lib', 'node')];
+  // $PREFIX/lib/node, where $PREFIX is the root of the Node.js installation.
+  var prefixDir;
+  // process.execPath is $PREFIX/bin/node except on Windows where it is
+  // $PREFIX\node.exe.
+  if (isWindows) {
+    prefixDir = path.resolve(process.execPath, '..');
+  } else {
+    prefixDir = path.resolve(process.execPath, '..', '..');
+  }
+  var paths = [path.resolve(prefixDir, 'lib', 'node')];
 
   if (homeDir) {
     paths.unshift(path.resolve(homeDir, '.node_libraries'));
@@ -636,11 +725,6 @@ Module._initPaths = function() {
   Module.globalPaths = modulePaths.slice(0);
 };
 
-// TODO(bnoordhuis) Unused, remove in the future.
-Module.requireRepl = internalUtil.deprecate(function() {
-  return NativeModule.require('internal/repl');
-}, 'Module.requireRepl is deprecated.');
-
 Module._preloadModules = function(requests) {
   if (!Array.isArray(requests))
     return;
@@ -656,9 +740,8 @@ Module._preloadModules = function(requests) {
       throw e;
     }
   }
-  requests.forEach(function(request) {
-    parent.require(request);
-  });
+  for (var n = 0; n < requests.length; n++)
+    parent.require(requests[n]);
 };
 
 Module._initPaths();
