@@ -1,11 +1,50 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 
+const async_wrap = process.binding('async_wrap');
 const TimerWrap = process.binding('timer_wrap').Timer;
 const L = require('internal/linkedlist');
+const internalUtil = require('internal/util');
+const { createPromise, promiseResolve } = process.binding('util');
 const assert = require('assert');
 const util = require('util');
 const debug = util.debuglog('timer');
 const kOnTimeout = TimerWrap.kOnTimeout | 0;
+// Two arrays that share state between C++ and JS.
+const { async_hook_fields, async_id_fields } = async_wrap;
+const {
+  initTriggerId,
+  // The needed emit*() functions.
+  emitInit,
+  emitBefore,
+  emitAfter,
+  emitDestroy
+} = require('async_hooks');
+// Grab the constants necessary for working with internal arrays.
+const { kInit, kDestroy, kAsyncIdCounter } = async_wrap.constants;
+// Symbols for storing async id state.
+const async_id_symbol = Symbol('asyncId');
+const trigger_async_id_symbol = Symbol('triggerAsyncId');
 
 // Timeout values > TIMEOUT_MAX are set to 1.
 const TIMEOUT_MAX = 2147483647; // 2^31-1
@@ -72,7 +111,7 @@ const TIMEOUT_MAX = 2147483647; // 2^31-1
 // timeout later, thus only needing to be appended.
 // Removal from an object-property linked list is also virtually constant-time
 // as can be seen in the lib/internal/linkedlist.js implementation.
-// Timeouts only need to process any timers due to currently timeout, which will
+// Timeouts only need to process any timers currently due to expire, which will
 // always be at the beginning of the list for reasons stated above. Any timers
 // after the first one encountered that does not yet need to timeout will also
 // always be due to timeout at a later time.
@@ -131,6 +170,16 @@ function insert(item, unrefed) {
     lists[msecs] = list = createTimersList(msecs, unrefed);
   }
 
+  if (!item[async_id_symbol] || item._destroyed) {
+    item._destroyed = false;
+    item[async_id_symbol] = ++async_id_fields[kAsyncIdCounter];
+    item[trigger_async_id_symbol] = initTriggerId();
+    if (async_hook_fields[kInit] > 0)
+      emitInit(
+        item[async_id_symbol], 'Timeout', item[trigger_async_id_symbol], item
+      );
+  }
+
   L.append(list, item);
   assert(!L.isEmpty(list)); // list is not empty
 }
@@ -183,7 +232,7 @@ function listOnTimeout() {
     if (diff < msecs) {
       var timeRemaining = msecs - (TimerWrap.now() - timer._idleStart);
       if (timeRemaining < 0) {
-        timeRemaining = 0;
+        timeRemaining = 1;
       }
       this.start(timeRemaining);
       debug('%d list wait because diff is %d', msecs, diff);
@@ -195,7 +244,14 @@ function listOnTimeout() {
     L.remove(timer);
     assert(timer !== L.peek(list));
 
-    if (!timer._onTimeout) continue;
+    if (!timer._onTimeout) {
+      if (async_hook_fields[kDestroy] > 0 && !timer._destroyed &&
+            typeof timer[async_id_symbol] === 'number') {
+        emitDestroy(timer[async_id_symbol]);
+        timer._destroyed = true;
+      }
+      continue;
+    }
 
     var domain = timer.domain;
     if (domain) {
@@ -245,11 +301,25 @@ function listOnTimeout() {
 // 4.7) what is in this smaller function.
 function tryOnTimeout(timer, list) {
   timer._called = true;
+  const timerAsyncId = (typeof timer[async_id_symbol] === 'number') ?
+    timer[async_id_symbol] : null;
   var threw = true;
+  if (timerAsyncId !== null)
+    emitBefore(timerAsyncId, timer[trigger_async_id_symbol]);
   try {
     ontimeout(timer);
     threw = false;
   } finally {
+    if (timerAsyncId !== null) {
+      if (!threw)
+        emitAfter(timerAsyncId);
+      if (!timer._repeat && async_hook_fields[kDestroy] > 0 &&
+          !timer._destroyed) {
+        emitDestroy(timerAsyncId);
+        timer._destroyed = true;
+      }
+    }
+
     if (!threw) return;
 
     // Postpone all later list events to next tick. We need to do this
@@ -301,6 +371,15 @@ function reuse(item) {
 
 // Remove a timer. Cancels the timeout and resets the relevant timer properties.
 const unenroll = exports.unenroll = function(item) {
+  // Fewer checks may be possible, but these cover everything.
+  if (async_hook_fields[kDestroy] > 0 &&
+      item &&
+      typeof item[async_id_symbol] === 'number' &&
+      !item._destroyed) {
+    emitDestroy(item[async_id_symbol]);
+    item._destroyed = true;
+  }
+
   var handle = reuse(item);
   if (handle) {
     debug('unenroll: list empty');
@@ -343,7 +422,7 @@ exports.enroll = function(item, msecs) {
  */
 
 
-exports.setTimeout = function(callback, after, arg1, arg2, arg3) {
+function setTimeout(callback, after, arg1, arg2, arg3) {
   if (typeof callback !== 'function') {
     throw new TypeError('"callback" argument must be a function');
   }
@@ -362,12 +441,20 @@ exports.setTimeout = function(callback, after, arg1, arg2, arg3) {
   }
 
   return createSingleTimeout(callback, after, args);
+}
+
+setTimeout[internalUtil.promisify.custom] = function(after, value) {
+  const promise = createPromise();
+  createSingleTimeout(promise, after, [value]);
+  return promise;
 };
 
+exports.setTimeout = setTimeout;
+
 function createSingleTimeout(callback, after, args) {
   after *= 1; // coalesce to number or NaN
   if (!(after >= 1 && after <= TIMEOUT_MAX))
-    after = 1; // schedule on next tick, follows browser behaviour
+    after = 1; // schedule on next tick, follows browser behavior
 
   var timer = new Timeout(after, callback, args);
   if (process.domain)
@@ -382,21 +469,23 @@ function createSingleTimeout(callback, after, args) {
 function ontimeout(timer) {
   var args = timer._timerArgs;
   var callback = timer._onTimeout;
+  if (typeof callback !== 'function')
+    return promiseResolve(callback, args[0]);
   if (!args)
-    callback.call(timer);
+    timer._onTimeout();
   else {
     switch (args.length) {
       case 1:
-        callback.call(timer, args[0]);
+        timer._onTimeout(args[0]);
         break;
       case 2:
-        callback.call(timer, args[0], args[1]);
+        timer._onTimeout(args[0], args[1]);
         break;
       case 3:
-        callback.call(timer, args[0], args[1], args[2]);
+        timer._onTimeout(args[0], args[1], args[2]);
         break;
       default:
-        callback.apply(timer, args);
+        Function.prototype.apply.call(callback, timer, args);
     }
   }
   if (timer._repeat)
@@ -454,7 +543,7 @@ exports.setInterval = function(callback, repeat, arg1, arg2, arg3) {
 function createRepeatTimeout(callback, repeat, args) {
   repeat *= 1; // coalesce to number or NaN
   if (!(repeat >= 1 && repeat <= TIMEOUT_MAX))
-    repeat = 1; // schedule on next tick, follows browser behaviour
+    repeat = 1; // schedule on next tick, follows browser behavior
 
   var timer = new Timeout(repeat, callback, args);
   timer._repeat = repeat;
@@ -483,6 +572,13 @@ function Timeout(after, callback, args) {
   this._onTimeout = callback;
   this._timerArgs = args;
   this._repeat = null;
+  this._destroyed = false;
+  this[async_id_symbol] = ++async_id_fields[kAsyncIdCounter];
+  this[trigger_async_id_symbol] = initTriggerId();
+  if (async_hook_fields[kInit] > 0)
+    emitInit(
+      this[async_id_symbol], 'Timeout', this[trigger_async_id_symbol], this
+    );
 }
 
 
@@ -537,6 +633,15 @@ Timeout.prototype.ref = function() {
 Timeout.prototype.close = function() {
   this._onTimeout = null;
   if (this._handle) {
+    // Fewer checks may be possible, but these cover everything.
+    if (async_hook_fields[kDestroy] > 0 &&
+        this &&
+        typeof this[async_id_symbol] === 'number' &&
+        !this._destroyed) {
+      emitDestroy(this[async_id_symbol]);
+      this._destroyed = true;
+    }
+
     this._idleTimeout = -1;
     this._handle[kOnTimeout] = null;
     this._handle.close();
@@ -640,11 +745,21 @@ function processImmediate() {
 // 4.7) what is in this smaller function.
 function tryOnImmediate(immediate, oldTail) {
   var threw = true;
+  emitBefore(immediate[async_id_symbol], immediate[trigger_async_id_symbol]);
   try {
-    // make the actual call outside the try/catch to allow it to be optimized
+    // make the actual call outside the try/finally to allow it to be optimized
     runCallback(immediate);
     threw = false;
   } finally {
+    // clearImmediate checks _callback === null for kDestroy hooks.
+    immediate._callback = null;
+    if (!threw)
+      emitAfter(immediate[async_id_symbol]);
+    if (async_hook_fields[kDestroy] > 0 && !immediate._destroyed) {
+      emitDestroy(immediate[async_id_symbol]);
+      immediate._destroyed = true;
+    }
+
     if (threw && immediate._idleNext) {
       // Handle any remaining on next tick, assuming we're still alive to do so.
       const curHead = immediateQueue.head;
@@ -666,6 +781,8 @@ function tryOnImmediate(immediate, oldTail) {
 function runCallback(timer) {
   const argv = timer._argv;
   const argc = argv ? argv.length : 0;
+  if (typeof timer._callback !== 'function')
+    return promiseResolve(timer._callback, argv[0]);
   switch (argc) {
     // fast-path callbacks with 0-3 arguments
     case 0:
@@ -678,7 +795,7 @@ function runCallback(timer) {
       return timer._callback(argv[0], argv[1], argv[2]);
     // more than 3 arguments run slower with .apply
     default:
-      return timer._callback.apply(timer, argv);
+      return Function.prototype.apply.call(timer._callback, timer, argv);
   }
 }
 
@@ -691,10 +808,17 @@ function Immediate() {
   this._callback = null;
   this._argv = null;
   this._onImmediate = null;
+  this._destroyed = false;
   this.domain = process.domain;
+  this[async_id_symbol] = ++async_id_fields[kAsyncIdCounter];
+  this[trigger_async_id_symbol] = initTriggerId();
+  if (async_hook_fields[kInit] > 0)
+    emitInit(
+      this[async_id_symbol], 'Immediate', this[trigger_async_id_symbol], this
+    );
 }
 
-exports.setImmediate = function(callback, arg1, arg2, arg3) {
+function setImmediate(callback, arg1, arg2, arg3) {
   if (typeof callback !== 'function') {
     throw new TypeError('"callback" argument must be a function');
   }
@@ -719,8 +843,16 @@ exports.setImmediate = function(callback, arg1, arg2, arg3) {
       break;
   }
   return createImmediate(args, callback);
+}
+
+setImmediate[internalUtil.promisify.custom] = function(value) {
+  const promise = createPromise();
+  createImmediate([value], promise);
+  return promise;
 };
 
+exports.setImmediate = setImmediate;
+
 function createImmediate(args, callback) {
   // declaring it `const immediate` causes v6.0.0 to deoptimize this function
   var immediate = new Immediate();
@@ -742,6 +874,13 @@ function createImmediate(args, callback) {
 exports.clearImmediate = function(immediate) {
   if (!immediate) return;
 
+  if (async_hook_fields[kDestroy] > 0 &&
+      immediate._callback !== null &&
+      !immediate._destroyed) {
+    emitDestroy(immediate[async_id_symbol]);
+    immediate._destroyed = true;
+  }
+
   immediate._onImmediate = null;
 
   immediateQueue.remove(immediate);
