@@ -1,8 +1,8 @@
 'use strict';
-var common = require('../common');
-var assert = require('assert');
-var net = require('net');
-var server = net.createServer(common.fail);
+const common = require('../common');
+const assert = require('assert');
+const net = require('net');
+const server = net.createServer(common.mustNotCall());
 
 common.refreshTmpDir();
 
